@@ -7,7 +7,6 @@ import shutil
 from importlib import import_module
 import json
 import os
-import pkg_resources
 import ssl
 import subprocess
 import sys
@@ -16,8 +15,9 @@ from typing import Any, Dict, List, Set, BinaryIO, Sequence
 import urllib.request
 
 import argcomplete
-import pystache
+import chevron
 
+from .convert import get_obj_repr_from_dict
 from .examples import (
     EXAMPLE_VINDALOO_CONF,
     EXAMPLE_BASE,
@@ -52,7 +52,7 @@ NEEDS_K8S_LOGIN = ('versions', 'deploy', 'build-push-deploy', 'edit-secret')
 CONFIG_DIR = 'k8s'
 CHECK_VERSION_URL = 'https://raw.githubusercontent.com/seznam/vindaloo/master/version.json'
 
-VERSION = '3.6.1'
+VERSION = '4.1.0'
 
 
 class RefreshException(Exception):
@@ -69,6 +69,7 @@ class Vindaloo:
         self.config_module = None  # konfigurace aktualne vybraneho prostredi (Dockerfily, deploymenty, porty, ...)
         self.args = None
         self.changed_secrets = {}  # Secrety naplanovane ke zmene
+        self.versions = {}  # Verze imagu
 
     def _am_i_logged_in(self) -> bool:
         """
@@ -117,7 +118,6 @@ class Vindaloo:
         k8s_clusters = k8s_clusters.split(',')
         docker_registry = self._input_text("Docker registry hostname: ")
         image_name = self._input_text("Name of docker repository (for example: avengers/web): ")
-        image_name = self._strip_image_name(image_name)
         ident_label = image_name.split('/')[1]
 
         os.mkdir(CONFIG_DIR)
@@ -160,7 +160,8 @@ class Vindaloo:
         # basic templates (Dockerfile, deployment, service)
         self._create_conf_file(
             '{}/Dockerfile'.format(templates_dir),
-            EXAMPLE_DOCKERFILE
+            EXAMPLE_DOCKERFILE,
+            dict(docker_registry=docker_registry),
         )
         self._create_conf_file(
             '{}/deployment.yaml'.format(templates_dir),
@@ -171,6 +172,13 @@ class Vindaloo:
             EXAMPLE_SERVICE
         )
 
+    def convert_manifest(self):
+        import yaml
+        with open(self.args.manifest, 'r') as fp:
+            manifest_data = yaml.load(fp, Loader=yaml.Loader)
+            res = get_obj_repr_from_dict(manifest_data)
+            self._out(res)
+
     def k8s_select_env(self) -> None:
         """
         Switch to selected K8S context.
@@ -191,7 +199,7 @@ class Vindaloo:
 
         self.config_module = self._import_config(dep_env)
         if not self.config_module:
-            self.fail("Environment '{}' does not have configuration.")
+            self.fail(f"Environment '{dep_env}' does not have configuration.")
 
         # prepneme se
         if not self.args.apply_output_dir:
@@ -201,9 +209,6 @@ class Vindaloo:
         for obj_type in K8S_OBJECT_TYPES:
             if obj_type not in self.config_module.K8S_OBJECTS:
                 continue  # Pokud tenhle typ nema tak jedeme dal
-            if obj_type == 'configmap':
-                self.deploy_configmaps(self.config_module.K8S_OBJECTS[obj_type])
-                continue
 
             for yaml_conf in self.config_module.K8S_OBJECTS[obj_type]:
                 if isinstance(yaml_conf, JsonSerializable):
@@ -213,7 +218,7 @@ class Vindaloo:
                     # pridame registry
                     yaml_conf['config']['registry'] = self.registry
 
-                    temp_file = self._create_file(yaml_conf['template'], yaml_conf['config'], from_templates=True)
+                    temp_file = self.create_file(yaml_conf['template'], yaml_conf['config'], from_templates=True)
                     ident = yaml_conf['config'].get('ident_label', 'unnamed')
 
                 if not temp_file:
@@ -235,7 +240,7 @@ class Vindaloo:
 
     def kubectl_apply(self, filename: str, name: str = 'unnamed', object_type: str = 'k8s_object') -> bool:
         """
-        Apply k8s yaml or save into output dir, when specified on command line.
+        Apply k8s JSON or save into output dir, when specified on command line.
         """
         if self.args.dryrun:
             self._out("CALL: kubectl apply -f {}".format(filename))
@@ -252,7 +257,7 @@ class Vindaloo:
 
             dest_filename = os.path.join(
                 self.args.apply_output_dir,
-                "{}{}_{}.yaml".format(prefix_, name, object_type)
+                "{}{}_{}.json".format(prefix_, name, object_type)
             )
             shutil.copy(filename, dest_filename)
             self._out("{} created.".format(dest_filename))
@@ -261,36 +266,6 @@ class Vindaloo:
             res = self.cmd(["kubectl", "apply", "-f", filename])
             return res.returncode == 0
 
-    def deploy_configmaps(self, configmaps):
-        """
-        For each configmap collects files and creates config map in k8s
-        """
-        for cm in configmaps:
-            name = cm['name']
-            files = [
-                (
-                    item['key'],
-                    self._create_file(item['file'], item.get('config', {}), from_templates=False, no_edit=True)
-                ) for item in cm['items']
-            ]
-            if files:
-
-                mapped = ["--from-file={}={}".format(x[0], x[1].name) for x in files]
-                self._out('Creating config map {}...'.format(name))
-                res = self.cmd(["kubectl", "create", "configmap", name, *mapped, "-o", "yaml", "--dry-run"], get_stdout=True)
-                assert res.returncode == 0
-
-                config_map_file = tempfile.NamedTemporaryFile("wb")
-                config_map_file.write(res.stdout)
-                config_map_file.seek(0)
-
-                # Optionally offers edit
-                if not self.args.noninteractive:
-                    if self._confirm("File {} was created. Do you want to modify it?".format(name), default="n"):
-                        self._open_in_editor(config_map_file)
-
-                assert self.kubectl_apply(config_map_file.name, name=name, object_type='configmap')
-
     def _import_envs_config(self) -> None:
         """
         Reads main configuration containing list of clusters and namespaces.
@@ -315,8 +290,7 @@ class Vindaloo:
         if not os.path.isfile("{}/{}.py".format(CONFIG_DIR, env)):
             return None
 
-        versions = json.load(open('{}/versions.json'.format(CONFIG_DIR)))
-        sys.modules['versions'] = versions
+        self.versions = json.load(open('{}/versions.json'.format(CONFIG_DIR)))
         sys.modules['vindaloo'].app = self
         sys.path.insert(0, os.path.abspath(CONFIG_DIR))
         try:
@@ -325,7 +299,9 @@ class Vindaloo:
                 del sys.modules[env]
             res_mod = import_module(env)
             return res_mod
-        except ModuleNotFoundError:
+        except ModuleNotFoundError as ex:
+            if env not in str(ex):
+                self.fail(f"Error importing env configuration: {ex}")
             return None
         finally:
             sys.path = sys.path[1:]
@@ -675,34 +651,30 @@ class Vindaloo:
             assert self._cmd_check(["kubectl", "config", "use-context", context], self.args.quiet)
             self._out("Environment changed to {} ({})".format(env, context))
 
-    def _create_file(
+    def create_file(
             self,
             template_file_name: str,
             conf: Dict,
             force_dest_file: str = None,
-            from_templates: bool=False,
+            from_templates: bool = False,
             no_edit=False
     ) -> BinaryIO:
         """
         Creates Dockerfile/yaml file using given template and config dict.
         """
-        data = ""
         if from_templates:
             src_file = "{}/templates/{}".format(CONFIG_DIR, template_file_name)
         else:
             src_file = "{}/{}".format(CONFIG_DIR, template_file_name)
 
         with open(src_file, "r") as template_file:
-            renderer = pystache.Renderer()
-            # Parse the tamplate
-            template = pystache.parse(template_file.read())
             # render using given variables
-            data = renderer.render(template, conf)
+            data = chevron.render(template_file, conf)
 
         if force_dest_file:
-            temp_file = open(force_dest_file, "wb")
+            temp_file = open(force_dest_file, "w+b")
         else:
-            temp_file = tempfile.NamedTemporaryFile("wb")
+            temp_file = tempfile.NamedTemporaryFile("w+b")
 
         temp_file.write(bytes(data, 'utf-8'))
         temp_file.seek(0)
@@ -728,7 +700,10 @@ class Vindaloo:
         else:
             temp_file = tempfile.NamedTemporaryFile("wb")
 
-        temp_file.write(bytes(obj.to_json(self), 'utf-8'))
+        temp_file.write(bytes(
+            json.dumps(obj.serialize(app=self), indent=4),
+            'utf-8'
+        ))
         temp_file.seek(0)
 
         # Optionally offers edit
@@ -755,9 +730,7 @@ class Vindaloo:
             for key, rel_path in conf['includes'].items():
                 assert os.path.exists(rel_path)
                 with open(rel_path, "r") as include_file:
-                    renderer = pystache.Renderer()
-                    template = pystache.parse(include_file.read())
-                    data = renderer.render(template, conf['config'])
+                    data = chevron.render(include_file, conf['config'])
                     new_context.setdefault('includes', {})[key] = data
 
         # Append config
@@ -773,7 +746,7 @@ class Vindaloo:
         # config with includes
         tmp_config = self._get_enriched_config_context(conf)
 
-        self._create_file(conf['template'], tmp_config, force_dest_file="Dockerfile", from_templates=True)
+        self.create_file(conf['template'], tmp_config, force_dest_file="Dockerfile", from_templates=True)
 
     def _check_version(self):
         try:
@@ -932,7 +905,9 @@ class Vindaloo:
 
         if command == "init":
             self.init_env()
-        if command == "build":
+        elif command == "convert-manifest":
+            self.convert_manifest()
+        elif command == "build":
             self.build_images()
         elif command == "pull":
             self.pull_images()
@@ -996,6 +971,9 @@ class Vindaloo:
         init_parser = subparsers.add_parser('init', help='prepares project for Vindaloo')
         init_parser.add_argument('dir', help='project directory')
 
+        convert_parser = subparsers.add_parser('convert-manifest', help='convert yaml manifest into Vindaloo config')
+        convert_parser.add_argument('manifest', help='manifest file')
+
         build_parser = subparsers.add_parser('build', help='builds Docker images (all of them)')
         build_parser.add_argument(
             'environment', help='environment for which we want to build image',
@@ -1169,3 +1147,6 @@ class Vindaloo:
 def run():
     tool = Vindaloo()
     tool.main()
+
+
+app = Vindaloo()
